@@ -22,7 +22,6 @@
 
 package org.xcsoar;
 
-import java.lang.reflect.Method;
 import android.app.Service;
 import android.app.Notification;
 import android.app.NotificationChannel;
@@ -83,9 +82,6 @@ public class MyService extends Service {
   }
 
   private static Notification createNotification(Context context, PendingIntent intent) {
-    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB)
-      return createNotificationOld(context, intent);
-
     Notification.Builder builder = new Notification.Builder(context)
       .setOngoing(true)
       .setContentIntent(intent)
@@ -99,27 +95,6 @@ public class MyService extends Service {
     return builder.build();
   }
 
-  private static Notification createNotificationOld(Context context, PendingIntent intent) {
-    Notification notification = new Notification(R.drawable.notification_icon, null,
-                                                 System.currentTimeMillis());
-
-    try {
-      Method method = Notification.class.getMethod("setLatestEventInfo",
-                                                   Context.class,
-                                                   CharSequence.class,
-                                                   CharSequence.class,
-                                                   PendingIntent.class);
-      method.invoke(notification, context, "XCSoar", "XCSoar is running",
-                    intent);
-    } catch (Exception e) {
-      /* ignore silently - shouldn't happen, but there's nothing we
-         can do about this */
-    }
-
-    notification.flags |= Notification.FLAG_ONGOING_EVENT;
-    return notification;
-  }
-
   private void onStart() {
     /* add an icon to the notification area while XCSoar runs, to
        remind the user that we're sucking his battery empty */
