@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@ Copyright_License {
 */
 
 #include "Bitmap.hpp"
-#include "Java/Class.hxx"
+#include "java/Class.hxx"
 
 jmethodID AndroidBitmap::recycle_method;
 jmethodID AndroidBitmap::getWidth_method;
