@@ -26,12 +26,14 @@ Copyright_License {
 
 #include "java/Object.hxx"
 #include "java/Class.hxx"
+#include "java/Closeable.hxx"
 #include "util/Compiler.h"
 
 #include <jni.h>
 #include <vector>
 
 class Context;
+class SensorListener;
 
 /**
  * Consolidated class for handling Java objects that work with Android GPS
@@ -58,11 +60,12 @@ public:
 
  private:
   // Java objects working with the GPS and the other sensors respectively.
-  Java::GlobalObject obj_InternalGPS_;
+  Java::GlobalCloseable internal_gps;
   Java::GlobalObject obj_NonGPSSensors_;
   std::vector<int> subscribable_sensors_;
 
-  InternalSensors(JNIEnv* env, jobject gps_obj, jobject sensors_obj);
+  InternalSensors(const Java::LocalObject &gps_obj,
+                  const Java::LocalObject &sensors_obj) noexcept;
   void getSubscribableSensors(JNIEnv* env, jobject sensors_obj);
  public:
   ~InternalSensors();
@@ -88,7 +91,7 @@ public:
 
   gcc_malloc
   static InternalSensors *create(JNIEnv* env, Context* native_view,
-                                 unsigned int index);
+                                 SensorListener &listener);
 };
 
 #endif
