@@ -93,6 +93,7 @@ FlyingComputer::Check(FlyingState &state, double time)
       state.flying = true;
       state.takeoff_time = moving_since;
       state.takeoff_location = moving_at;
+      state.takeoff_altitude = moving_altitude;
       state.flight_time = 0;
 
       /* when a new flight starts, forget the old release and power-on/off time */
@@ -126,7 +127,7 @@ FlyingComputer::Check(FlyingState &state, double time)
 
 void
 FlyingComputer::Moving(FlyingState &state, double time, double dt,
-                       const GeoPoint &location)
+                       const GeoPoint &location, double altitude) noexcept
 {
   // Increase InFlight countdown for further evaluation
   moving_clock.Add(dt);
@@ -134,6 +135,7 @@ FlyingComputer::Moving(FlyingState &state, double time, double dt,
   if (moving_since < 0) {
     moving_since = time;
     moving_at = location;
+    moving_altitude = altitude;
   }
 
   // We are moving so we are certainly not on the ground
@@ -269,13 +271,13 @@ FlyingComputer::Compute(double takeoff_speed,
   const auto any_altitude = basic.GetAnyAltitude();
 
   if (!basic.airspeed_available && !calculated.altitude_agl_valid &&
-      any_altitude.first && last_ground_altitude >= 0 &&
-      any_altitude.second > last_ground_altitude + 250) {
+      any_altitude && last_ground_altitude >= 0 &&
+      *any_altitude > last_ground_altitude + 250) {
     /* lower the threshold for "not moving" when the aircraft is high
        above the take-off airfield and there's no airspeed probe; this
        shall reduce the risk of false landing detection when flying in
        strong head wind (e.g. ridge or wave) */
-    auto dh = any_altitude.second - last_ground_altitude;
+    auto dh = *any_altitude - last_ground_altitude;
 
     if (dh > 1000)
       takeoff_speed /= 4;
@@ -287,20 +289,21 @@ FlyingComputer::Compute(double takeoff_speed,
 
   if (CheckTakeOffSpeed(takeoff_speed, basic) ||
       CheckAltitudeAGL(calculated))
-    Moving(flying, basic.time, dt, basic.location);
+    Moving(flying, basic.time, dt, basic.location,
+           basic.GetAnyAltitude().value_or(0));
   else if (!flying.flying ||
            (CheckLandingSpeed(takeoff_speed, basic) &&
-            (!any_altitude.first || !CheckClimbing(dt, any_altitude.second))))
+            (!any_altitude || !CheckClimbing(dt, *any_altitude))))
     Stationary(flying, basic.time, dt, basic.location);
 
   if (basic.engine_noise_level_available)
     CheckPowered(dt, basic, flying);
 
-  if (any_altitude.first) {
+  if (any_altitude) {
     if (flying.on_ground)
-      last_ground_altitude = any_altitude.second;
+      last_ground_altitude = *any_altitude;
 
-    CheckRelease(flying, basic.time, basic.location, any_altitude.second);
+    CheckRelease(flying, basic.time, basic.location, *any_altitude);
   } else
     sinking_since = -1;
 
@@ -322,7 +325,7 @@ FlyingComputer::Compute(double takeoff_speed,
     return;
 
   if (state.ground_speed > takeoff_speed)
-    Moving(flying, state.time, dt, state.location);
+    Moving(flying, state.time, dt, state.location, state.altitude);
   else
     Stationary(flying, state.time, dt, state.location);
 }
