@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -33,25 +33,21 @@ Copyright_License {
 #include "Form/DataField/Listener.hpp"
 #include "Form/DataField/Prefix.hpp"
 #include "Engine/Airspace/Airspaces.hpp"
-#include "Engine/Airspace/AbstractAirspace.hpp"
 #include "Renderer/AirspaceListRenderer.hpp"
 #include "Renderer/TwoTextRowsRenderer.hpp"
 #include "Look/MapLook.hpp"
 #include "Look/DialogLook.hpp"
-#include "Screen/Canvas.hpp"
-#include "Screen/Layout.hpp"
-#include "Screen/Key.h"
-#include "Compiler.h"
-#include "Util/Macros.hpp"
+#include "util/Compiler.h"
+#include "util/Macros.hpp"
 #include "Units/Units.hpp"
 #include "Formatter/AngleFormatter.hpp"
 #include "UIGlobals.hpp"
 #include "Interface.hpp"
-#include "Blackboard/ScopeGPSListener.hpp"
+#include "Blackboard/BlackboardListener.hpp"
 #include "Language/Language.hpp"
+#include "util/StringCompare.hxx"
 
-#include <assert.h>
-#include <stdlib.h>
+#include <cassert>
 #include <stdio.h>
 
 enum Controls {
@@ -61,15 +57,11 @@ enum Controls {
   TYPE,
 };
 
-enum Buttons {
-  DETAILS,
-};
-
 class AirspaceFilterWidget;
 
 class AirspaceListWidget final
   : public ListWidget, public DataFieldListener,
-    public ActionListener, NullBlackboardListener {
+    NullBlackboardListener {
   AirspaceFilterWidget &filter_widget;
 
   AirspaceSelectInfoVector items;
@@ -84,42 +76,39 @@ public:
   void FilterMode(bool direction);
   void OnAirspaceListEnter(unsigned index);
 
-  /* virtual methods from class Widget */
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override;
-
-  virtual void Unprepare() override {
-    DeleteWindow();
+  void ShowDetails() noexcept {
+    OnAirspaceListEnter(GetList().GetCursorIndex());
   }
 
-  virtual void Show(const PixelRect &rc) override {
+  /* virtual methods from class Widget */
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override;
+
+  void Show(const PixelRect &rc) noexcept override {
     ListWidget::Show(rc);
     UpdateList();
     CommonInterface::GetLiveBlackboard().AddListener(*this);
   }
 
-  virtual void Hide() override {
+  void Hide() noexcept override {
     CommonInterface::GetLiveBlackboard().RemoveListener(*this);
 
     ListWidget::Hide();
   }
 
   /* virtual methods from ListItemRenderer */
-  virtual void OnPaintItem(Canvas &canvas, const PixelRect rc,
-                           unsigned idx) override;
+  void OnPaintItem(Canvas &canvas, const PixelRect rc,
+                   unsigned idx) noexcept override;
 
   /* virtual methods from ListCursorHandler */
-  virtual bool CanActivateItem(unsigned index) const override {
+  bool CanActivateItem(unsigned index) const noexcept override {
     return true;
   }
 
-  virtual void OnActivateItem(unsigned index) override;
-
-  /* virtual methods from ActionListener */
-  virtual void OnAction(int id) override;
+  void OnActivateItem(unsigned index) noexcept override;
 
   /* virtual methods from DataFieldListener */
-  virtual void OnModified(DataField &df) override;
+  void OnModified(DataField &df) noexcept override;
 
 private:
   /* virtual methods from BlackboardListener */
@@ -140,29 +129,29 @@ public:
   void Update();
 
   /* virtual methods from class Widget */
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override;
-#ifdef GNAV
-  virtual bool KeyPress(unsigned key_code) override;
-#endif
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override;
 };
 
 class AirspaceListButtons final : public RowFormWidget {
-  ActionListener &dialog;
-  ActionListener *list;
+  WndForm &dialog;
+  AirspaceListWidget *list;
 
 public:
-  AirspaceListButtons(const DialogLook &look, ActionListener &_dialog)
+  AirspaceListButtons(const DialogLook &look, WndForm &_dialog) noexcept
     :RowFormWidget(look), dialog(_dialog) {}
 
-  void SetList(ActionListener *_list) {
+  void SetList(AirspaceListWidget *_list) {
     list = _list;
   }
 
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override {
-    AddButton(_("Details"), *list, DETAILS);
-    AddButton(_("Close"), dialog, mrCancel);
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override {
+    AddButton(_("Details"), [this](){
+      list->ShowDetails();
+    });
+
+    AddButton(_("Close"), dialog.MakeModalResultCallback(mrCancel));
   }
 };
 
@@ -200,12 +189,12 @@ static constexpr StaticEnumChoice type_filter_list[] = {
 
 struct AirspaceListWidgetState
 {
-  fixed distance;
+  double distance;
   unsigned direction;
   unsigned type;
 
   AirspaceListWidgetState()
-    :distance(fixed(-1)), direction(WILDCARD), type(WILDCARD) {}
+    :distance(-1), direction(WILDCARD), type(WILDCARD) {}
 };
 
 static AirspaceListWidgetState dialog_state;
@@ -224,21 +213,11 @@ AirspaceListWidget::OnAirspaceListEnter(unsigned i)
 }
 
 void
-AirspaceListWidget::OnActivateItem(unsigned index)
+AirspaceListWidget::OnActivateItem(unsigned index) noexcept
 {
   OnAirspaceListEnter(index);
 }
 
-void
-AirspaceListWidget::OnAction(int id)
-{
-  switch (Buttons(id)) {
-  case DETAILS:
-    OnAirspaceListEnter(GetList().GetCursorIndex());
-    break;
-  }
-}
-
 void
 AirspaceListWidget::UpdateList()
 {
@@ -258,7 +237,7 @@ AirspaceListWidget::UpdateList()
       : Angle::Degrees(dialog_state.direction);
   }
 
-  if (positive(dialog_state.distance))
+  if (dialog_state.distance > 0)
     data.distance = dialog_state.distance;
 
   items = FilterAirspaces(*airspaces,
@@ -270,7 +249,8 @@ AirspaceListWidget::UpdateList()
 }
 
 void
-AirspaceListWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
+AirspaceListWidget::Prepare(ContainerWindow &parent,
+                            const PixelRect &rc) noexcept
 {
   const DialogLook &look = UIGlobals::GetDialogLook();
   CreateList(parent, look, rc,
@@ -283,7 +263,7 @@ inline void
 AirspaceListWidget::FilterMode(bool direction)
 {
   if (direction) {
-    dialog_state.distance = fixed(-1);
+    dialog_state.distance = -1;
     dialog_state.direction = WILDCARD;
 
     filter_widget.LoadValueEnum(DISTANCE, WILDCARD);
@@ -294,13 +274,13 @@ AirspaceListWidget::FilterMode(bool direction)
 }
 
 void
-AirspaceListWidget::OnModified(DataField &df)
+AirspaceListWidget::OnModified(DataField &df) noexcept
 {
   if (filter_widget.IsDataField(DISTANCE, df)) {
     DataFieldEnum &dfe = (DataFieldEnum &)df;
     dialog_state.distance = dfe.GetValue() != WILDCARD
-      ? Units::ToSysDistance(fixed(dfe.GetValue()))
-      : fixed(-1);
+      ? Units::ToSysDistance(dfe.GetValue())
+      : -1.;
 
   } else if (filter_widget.IsDataField(DIRECTION, df)) {
     DataFieldEnum &dfe = (DataFieldEnum &)df;
@@ -317,13 +297,12 @@ AirspaceListWidget::OnModified(DataField &df)
 
 void
 AirspaceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
-                                unsigned i)
+                                unsigned i) noexcept
 {
   if (items.empty()) {
     assert(i == 0);
 
-    canvas.DrawText(rc.left + Layout::GetTextPadding(),
-                    rc.top + Layout::GetTextPadding(), _("No Match!"));
+    row_renderer.DrawFirstRow(canvas, rc, _("No Match!"));
     return;
   }
 
@@ -356,7 +335,7 @@ AirspaceListWidget::OnGPSUpdate(const MoreData &basic)
   if (dialog_state.direction == 0 && !CommonInterface::Calculated().circling) {
     const Angle heading = basic.attitude.heading;
     Angle a = last_heading - heading;
-    if (a.AsDelta().AbsoluteDegrees() >= fixed(10)) {
+    if (a.AsDelta().AbsoluteDegrees() >= 10) {
       last_heading = heading;
 
       UpdateList();
@@ -377,31 +356,6 @@ AirspaceFilterWidget::Update()
   direction_control.RefreshDisplay();
 }
 
-#ifdef GNAV
-
-bool
-AirspaceFilterWidget::KeyPress(unsigned key_code)
-{
-  switch (key_code) {
-  case KEY_APP1:
-    LoadValueEnum(TYPE, WILDCARD);
-    return true;
-
-  case KEY_APP2:
-    LoadValueEnum(TYPE, RESTRICT);
-    return true;
-
-  case KEY_APP3:
-    LoadValueEnum(TYPE, PROHIBITED);
-    return true;
-
-  default:
-    return false;
-  }
-}
-
-#endif /* GNAV */
-
 static void
 FillDistanceEnum(DataFieldEnum &df)
 {
@@ -463,7 +417,7 @@ CreateDirectionDataField(DataFieldListener *listener)
 
 void
 AirspaceFilterWidget::Prepare(ContainerWindow &parent,
-                              const PixelRect &rc)
+                              const PixelRect &rc) noexcept
 {
   Add(_("Name"), nullptr, CreateNameDataField(listener));
   Add(_("Distance"), nullptr, CreateDistanceDataField(listener));
@@ -481,24 +435,24 @@ ShowAirspaceListDialog(const Airspaces &_airspaces,
   airspaces = &_airspaces;
   location = CommonInterface::Basic().location;
 
-  WidgetDialog dialog(look);
+  WidgetDialog dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+                      look, _("Select Airspace"));
 
-  AirspaceFilterWidget *filter_widget = new AirspaceFilterWidget(look);
+  auto filter_widget = std::make_unique<AirspaceFilterWidget>(look);
 
-  AirspaceListButtons *buttons_widget = new AirspaceListButtons(look, dialog);
+  auto list_widget = std::make_unique<AirspaceListWidget>(*filter_widget);
 
-  TwoWidgets *left_widget =
-    new TwoWidgets(filter_widget, buttons_widget, true);
+  auto buttons_widget = std::make_unique<AirspaceListButtons>(look, dialog);
 
-  AirspaceListWidget *const list_widget =
-    new AirspaceListWidget(*filter_widget);
+  filter_widget->SetListener(list_widget.get());
+  buttons_widget->SetList(list_widget.get());
 
-  filter_widget->SetListener(list_widget);
-  buttons_widget->SetList(list_widget);
+  auto left_widget = std::make_unique<TwoWidgets>(std::move(filter_widget),
+                                                  std::move(buttons_widget),
+                                                  true);
 
-  TwoWidgets *widget = new TwoWidgets(left_widget, list_widget, false);
-
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("Select Airspace"), widget);
+  dialog.FinishPreliminary(new TwoWidgets(std::move(left_widget),
+                                          std::move(list_widget), false));
   dialog.ShowModal();
 }
 
