@@ -22,35 +22,19 @@
 */
 
 #include "DeviceEditWidget.hpp"
-#include "Dialogs/ComboPicker.hpp"
+#include "PortDataField.hpp"
+#include "PortPicker.hpp"
 #include "UIGlobals.hpp"
 #include "util/Compiler.h"
-#include "util/Macros.hpp"
 #include "util/NumberParser.hpp"
 #include "Language/Language.hpp"
 #include "Form/DataField/Enum.hpp"
 #include "Form/DataField/Boolean.hpp"
 #include "Form/DataField/String.hpp"
-#include "Form/DataField/ComboList.hpp"
 #include "Device/Register.hpp"
 #include "Device/Driver.hpp"
-#include "Device/Features.hpp"
 #include "Interface.hpp"
 
-#ifdef HAVE_POSIX
-#include "Device/Port/TTYEnumerator.hpp"
-#endif
-
-#ifdef ANDROID
-#include "java/Global.hxx"
-#include "java/Ref.hxx"
-#include "java/String.hxx"
-#include "Android/BluetoothHelper.hpp"
-#include "Android/UsbSerialHelper.hpp"
-#include "Device/Port/AndroidIOIOUartPort.hpp"
-#include "ScanBluetoothLeDialog.hpp"
-#endif
-
 enum ControlIndex {
   Port, BaudRate, BulkBaudRate,
   IP_ADDRESS,
@@ -60,238 +44,6 @@ enum ControlIndex {
   K6Bt,
 };
 
-static constexpr struct {
-  DeviceConfig::PortType type;
-  const TCHAR *label;
-} port_types[] = {
-  { DeviceConfig::PortType::DISABLED, N_("Disabled") },
-#ifdef HAVE_INTERNAL_GPS
-  { DeviceConfig::PortType::INTERNAL, N_("Built-in GPS & sensors") },
-#endif
-#ifdef ANDROID
-  { DeviceConfig::PortType::RFCOMM_SERVER, N_("Bluetooth server") },
-  { DeviceConfig::PortType::DROIDSOAR_V2, _T("DroidSoar V2") },
-  { DeviceConfig::PortType::GLIDER_LINK, _T("GliderLink traffic receiver") },
-#ifndef NDEBUG
-  { DeviceConfig::PortType::NUNCHUCK, N_("IOIO switches and Nunchuk") },
-#endif
-  { DeviceConfig::PortType::I2CPRESSURESENSOR, N_("IOIO I²C pressure sensor") },
-  { DeviceConfig::PortType::IOIOVOLTAGE, N_("IOIO voltage sensor") },
-#endif
-
-  { DeviceConfig::PortType::TCP_CLIENT, N_("TCP client") },
-
-  /* label not translated for now, until we have a TCP/UDP port
-     selection UI */
-  { DeviceConfig::PortType::TCP_LISTENER, N_("TCP port") },
-  { DeviceConfig::PortType::UDP_LISTENER, N_("UDP port") },
-
-  { DeviceConfig::PortType::SERIAL, NULL } /* sentinel */
-};
-
-/** the number of fixed port types (excludes Serial, Bluetooth and IOIOUart) */
-static constexpr unsigned num_port_types = ARRAY_SIZE(port_types) - 1;
-
-static unsigned
-AddPort(DataFieldEnum &df, DeviceConfig::PortType type,
-        const TCHAR *text, const TCHAR *display_string=NULL,
-        const TCHAR *help=NULL) noexcept
-{
-  /* the uppper 16 bit is the port type, and the lower 16 bit is a
-     serial number to make the enum id unique */
-
-  unsigned id = ((unsigned)type << 16) + df.Count();
-  df.AddChoice(id, text, display_string, help);
-  return id;
-}
-
-#if defined(HAVE_POSIX)
-
-static bool
-DetectSerialPorts(DataFieldEnum &df) noexcept
-{
-  TTYEnumerator enumerator;
-  if (enumerator.HasFailed())
-    return false;
-
-  unsigned sort_start = df.Count();
-
-  bool found = false;
-  const char *path;
-  while ((path = enumerator.Next()) != nullptr) {
-    const char *display_string = path;
-    if (memcmp(path, "/dev/", 5) == 0)
-      display_string = path + 5;
-
-    AddPort(df, DeviceConfig::PortType::SERIAL, path, display_string);
-    found = true;
-  }
-
-  if (found)
-    df.Sort(sort_start);
-
-  return found;
-}
-
-#endif
-
-#if defined(_WIN32) && !defined(HAVE_POSIX)
-
-static void
-FillDefaultSerialPorts(DataFieldEnum &df) noexcept
-{
-  for (unsigned i = 1; i <= 10; ++i) {
-    TCHAR buffer[64];
-    _stprintf(buffer, _T("COM%u:"), i);
-    AddPort(df, DeviceConfig::PortType::SERIAL, buffer);
-  }
-}
-
-#endif
-
-static void
-FillPortTypes(DataFieldEnum &df, const DeviceConfig &config) noexcept
-{
-  for (unsigned i = 0; port_types[i].label != NULL; i++) {
-    unsigned id = AddPort(df, port_types[i].type, port_types[i].label,
-                          gettext(port_types[i].label));
-
-    if (port_types[i].type == config.port_type)
-      df.Set(id);
-  }
-}
-
-static void
-SetPort(DataFieldEnum &df, DeviceConfig::PortType type,
-        const TCHAR *value) noexcept
-{
-  assert(value != NULL);
-
-  if (!df.Set(value))
-    df.Set(AddPort(df, type, value));
-}
-
-static void
-FillSerialPorts(DataFieldEnum &df, const DeviceConfig &config) noexcept
-{
-#if defined(HAVE_POSIX)
-  DetectSerialPorts(df);
-#elif defined(_WIN32)
-  FillDefaultSerialPorts(df);
-#endif
-
-  if (config.port_type == DeviceConfig::PortType::SERIAL)
-    SetPort(df, config.port_type, config.path);
-}
-
-static void
-FillAndroidBluetoothPorts(DataFieldEnum &df,
-                          const DeviceConfig &config) noexcept
-{
-#ifdef ANDROID
-  JNIEnv *env = Java::GetEnv();
-  jobjectArray bonded = BluetoothHelper::list(env);
-  if (bonded == NULL)
-    return;
-
-  jsize n = env->GetArrayLength(bonded) / 2;
-  for (jsize i = 0; i < n; ++i) {
-    jstring address = (jstring)env->GetObjectArrayElement(bonded, i * 2);
-    if (address == NULL)
-      continue;
-
-    const char *address2 = env->GetStringUTFChars(address, NULL);
-    if (address2 == NULL)
-      continue;
-
-    jstring name = (jstring)env->GetObjectArrayElement(bonded, i * 2 + 1);
-    const char *name2 = name != NULL
-      ? env->GetStringUTFChars(name, NULL)
-      : NULL;
-
-    AddPort(df, DeviceConfig::PortType::RFCOMM, address2, name2);
-
-    env->ReleaseStringUTFChars(address, address2);
-    if (name2 != NULL)
-      env->ReleaseStringUTFChars(name, name2);
-  }
-
-  env->DeleteLocalRef(bonded);
-
-  if (config.port_type == DeviceConfig::PortType::RFCOMM &&
-      !config.bluetooth_mac.empty())
-    SetPort(df, DeviceConfig::PortType::RFCOMM, config.bluetooth_mac);
-#endif
-}
-
-static void
-FillAndroidUsbSerialPorts(DataFieldEnum &df,
-                          const DeviceConfig &config) noexcept
-{
-#ifdef ANDROID
-  JNIEnv *env = Java::GetEnv();
-  Java::LocalRef<jobjectArray> list{env, UsbSerialHelper::list(env)};
-  if (!list)
-    return;
-
-  jsize n = env->GetArrayLength(list) / 2;
-  for (jsize i = 0; i < n; ++i) {
-    Java::String j_id{env, (jstring)env->GetObjectArrayElement(list, i * 2)};
-    if (!j_id)
-      continue;
-
-    Java::String j_name{env, (jstring)env->GetObjectArrayElement(list, i * 2 + 1)};
-    if (!j_name)
-      continue;
-
-    const auto id = j_id.ToString();
-    const auto name = j_name.ToString();
-
-    char display_string[256];
-    StringFormat(display_string, sizeof(display_string),
-                 "USB: %s", name.c_str());
-
-    AddPort(df, DeviceConfig::PortType::ANDROID_USB_SERIAL,
-            id.c_str(), display_string);
-  }
-
-  if (config.port_type == DeviceConfig::PortType::ANDROID_USB_SERIAL &&
-      !config.path.empty())
-    SetPort(df, DeviceConfig::PortType::ANDROID_USB_SERIAL, config.path);
-#endif
-}
-
-static void
-FillAndroidIOIOPorts(DataFieldEnum &df, const DeviceConfig &config) noexcept
-{
-#if defined(ANDROID)
-  df.EnableItemHelp(true);
-
-  TCHAR tempID[4];
-  TCHAR tempName[15];
-  for (unsigned i = 0; i < AndroidIOIOUartPort::getNumberUarts(); i++) {
-    StringFormatUnsafe(tempID, _T("%u"), i);
-    StringFormat(tempName, sizeof(tempName), _T("IOIO UART %u"), i);
-    unsigned id = AddPort(df, DeviceConfig::PortType::IOIOUART,
-                          tempID, tempName,
-                          AndroidIOIOUartPort::getPortHelp(i));
-    if (config.port_type == DeviceConfig::PortType::IOIOUART &&
-        config.ioio_uart_id == i)
-      df.Set(id);
-  }
-#endif
-}
-
-static void
-FillPorts(DataFieldEnum &df, const DeviceConfig &config) noexcept
-{
-  FillPortTypes(df, config);
-  FillSerialPorts(df, config);
-  FillAndroidBluetoothPorts(df, config);
-  FillAndroidUsbSerialPorts(df, config);
-  FillAndroidIOIOPorts(df, config);
-}
-
 static void
 FillBaudRates(DataFieldEnum &dfe) noexcept
 {
@@ -344,90 +96,18 @@ FillPress(DataFieldEnum &dfe) noexcept
   dfe.addEnumText(_T("Static"), (unsigned)DeviceConfig::PressureUse::STATIC_ONLY);
   dfe.addEnumText(_T("TE probe (compensated vario)"), (unsigned)DeviceConfig::PressureUse::TEK_PRESSURE);
   dfe.addEnumText(_T("Pitot (airspeed)"), (unsigned)DeviceConfig::PressureUse::PITOT);
-  dfe.addEnumText(_T("Pitot zero calibration"), (unsigned)DeviceConfig::PressureUse::PITOT_ZERO);
-}
-
-static void
-SetPort(DataFieldEnum &df, const DeviceConfig &config) noexcept
-{
-  switch (config.port_type) {
-  case DeviceConfig::PortType::DISABLED:
-  case DeviceConfig::PortType::AUTO:
-  case DeviceConfig::PortType::INTERNAL:
-  case DeviceConfig::PortType::DROIDSOAR_V2:
-  case DeviceConfig::PortType::NUNCHUCK:
-  case DeviceConfig::PortType::I2CPRESSURESENSOR:
-  case DeviceConfig::PortType::IOIOVOLTAGE:
-  case DeviceConfig::PortType::TCP_CLIENT:
-  case DeviceConfig::PortType::TCP_LISTENER:
-  case DeviceConfig::PortType::UDP_LISTENER:
-  case DeviceConfig::PortType::PTY:
-  case DeviceConfig::PortType::RFCOMM_SERVER:
-  case DeviceConfig::PortType::GLIDER_LINK:
-    break;
-
-  case DeviceConfig::PortType::SERIAL:
-  case DeviceConfig::PortType::ANDROID_USB_SERIAL:
-    SetPort(df, config.port_type, config.path);
-    return;
-
-  case DeviceConfig::PortType::RFCOMM:
-    SetPort(df, config.port_type, config.bluetooth_mac);
-    return;
-
-  case DeviceConfig::PortType::IOIOUART:
-    StaticString<16> buffer;
-    buffer.UnsafeFormat(_T("%d"), config.ioio_uart_id);
-    df.Set(buffer);
-    return;
-  }
-
-  for (unsigned i = 0; port_types[i].label != NULL; i++) {
-    if (port_types[i].type == config.port_type) {
-      df.Set(port_types[i].label);
-      break;
-    }
-  }
 }
 
 static bool
-EditPortCallback(const TCHAR *caption, DataField &_df,
+EditPortCallback(const TCHAR *caption, DataField &df,
                  const TCHAR *help_text) noexcept
 {
-  DataFieldEnum &df = (DataFieldEnum &)_df;
-
-  ComboList combo_list = df.CreateComboList(nullptr);
-
-#ifdef ANDROID
-  static constexpr int SCAN_BLUETOOTH_LE = -1;
-  if (BluetoothHelper::HasLe(Java::GetEnv()))
-    combo_list.Append(SCAN_BLUETOOTH_LE, _("Bluetooth LE"));
-#endif
-
-  int i = ComboPicker(caption, combo_list, help_text);
-  if (i < 0)
-    return false;
-
-  const ComboList::Item &item = combo_list[i];
-
-#ifdef ANDROID
-  if (item.int_value == SCAN_BLUETOOTH_LE) {
-    auto address = ScanBluetoothLeDialog();
-    if (address.empty())
-        return false;
-
-    SetPort(df, DeviceConfig::PortType::RFCOMM, address.c_str());
-    return true;
-  }
-#endif
-
-  df.SetFromCombo(item.int_value, item.string_value.c_str());
-  return true;
+  return PortPicker((DataFieldEnum &)df, caption);
 }
 
 DeviceEditWidget::DeviceEditWidget(const DeviceConfig &_config) noexcept
   :RowFormWidget(UIGlobals::GetDialogLook()),
-   config(_config), listener(NULL) {}
+   config(_config) {}
 
 void
 DeviceEditWidget::SetConfig(const DeviceConfig &_config) noexcept
@@ -516,11 +196,11 @@ static bool
 SupportsBulkBaudRate(const DataField &df) noexcept
 {
   const TCHAR *driver_name = df.GetAsString();
-  if (driver_name == NULL)
+  if (driver_name == nullptr)
     return false;
 
   const struct DeviceRegister *driver = FindDriverByName(driver_name);
-  if (driver == NULL)
+  if (driver == nullptr)
     return false;
 
   return driver->SupportsBulkBaudRate();
@@ -531,11 +211,11 @@ static bool
 CanReceiveSettings(const DataField &df) noexcept
 {
   const TCHAR *driver_name = df.GetAsString();
-  if (driver_name == NULL)
+  if (driver_name == nullptr)
     return false;
 
   const struct DeviceRegister *driver = FindDriverByName(driver_name);
-  if (driver == NULL)
+  if (driver == nullptr)
     return false;
 
   return driver->CanReceiveSettings();
@@ -546,29 +226,16 @@ static bool
 CanSendSettings(const DataField &df) noexcept
 {
   const TCHAR *driver_name = df.GetAsString();
-  if (driver_name == NULL)
+  if (driver_name == nullptr)
     return false;
 
   const struct DeviceRegister *driver = FindDriverByName(driver_name);
-  if (driver == NULL)
+  if (driver == nullptr)
     return false;
 
   return driver->CanSendSettings();
 }
 
-gcc_pure
-static DeviceConfig::PortType
-GetPortType(const DataField &df) noexcept
-{
-  const DataFieldEnum &dfe = (const DataFieldEnum &)df;
-  const unsigned port = dfe.GetValue();
-
-  if (port < num_port_types)
-    return port_types[port].type;
-
-  return (DeviceConfig::PortType)(port >> 16);
-}
-
 gcc_pure
 static bool
 CanPassThrough(const DataField &df) noexcept
@@ -587,9 +254,10 @@ CanPassThrough(const DataField &df) noexcept
 void
 DeviceEditWidget::UpdateVisibilities() noexcept
 {
-  const DeviceConfig::PortType type = GetPortType(GetDataField(Port));
+  const auto &port_df = (const DataFieldEnum &)GetDataField(Port);
+  const DeviceConfig::PortType type = GetPortType(port_df);
   const bool maybe_bluetooth =
-    DeviceConfig::MaybeBluetooth(type, GetDataField(Port).GetAsString());
+    DeviceConfig::MaybeBluetooth(type, port_df.GetAsString());
   const bool k6bt = maybe_bluetooth && GetValueBoolean(K6Bt);
   const bool uses_speed = DeviceConfig::UsesSpeed(type) || k6bt;
 
@@ -628,13 +296,13 @@ DeviceEditWidget::Prepare(ContainerWindow &parent,
 
   DataFieldEnum *port_df = new DataFieldEnum(this);
   FillPorts(*port_df, config);
-  auto *port_control = Add(_("Port"), NULL, port_df);
+  auto *port_control = Add(_("Port"), nullptr, port_df);
   port_control->SetEditCallback(EditPortCallback);
 
   DataFieldEnum *baud_rate_df = new DataFieldEnum(this);
   FillBaudRates(*baud_rate_df);
   baud_rate_df->Set(config.baud_rate);
-  Add(_("Baud rate"), NULL, baud_rate_df);
+  Add(_("Baud rate"), nullptr, baud_rate_df);
 
   DataFieldEnum *bulk_baud_rate_df = new DataFieldEnum(this);
   bulk_baud_rate_df->addEnumText(_T("Default"), 0u);
@@ -646,12 +314,12 @@ DeviceEditWidget::Prepare(ContainerWindow &parent,
 
   DataFieldString *ip_address_df = new DataFieldString(_T(""), this);
   ip_address_df->Set(config.ip_address);
-  Add(_("IP address"), NULL, ip_address_df);
+  Add(_("IP address"), nullptr, ip_address_df);
 
   DataFieldEnum *tcp_port_df = new DataFieldEnum(this);
   FillTCPPorts(*tcp_port_df);
   tcp_port_df->Set(config.tcp_port);
-  Add(_("TCP port"), NULL, tcp_port_df);
+  Add(_("TCP port"), nullptr, tcp_port_df);
 
   DataFieldEnum *i2c_bus_df = new DataFieldEnum(this);
   FillI2CBus(*i2c_bus_df);
@@ -678,13 +346,13 @@ DeviceEditWidget::Prepare(ContainerWindow &parent,
   DataFieldEnum *driver_df = new DataFieldEnum(this);
 
   const struct DeviceRegister *driver;
-  for (unsigned i = 0; (driver = GetDriverByIndex(i)) != NULL; i++)
+  for (unsigned i = 0; (driver = GetDriverByIndex(i)) != nullptr; i++)
     driver_df->addEnumText(driver->name, driver->display_name);
 
   driver_df->Sort(1);
   driver_df->Set(config.driver_name);
 
-  Add(_("Driver"), NULL, driver_df);
+  Add(_("Driver"), nullptr, driver_df);
 
   // for a passthrough device, offer additional driver
   AddBoolean(_("Passthrough device"),
@@ -766,6 +434,8 @@ FinishPortField(DeviceConfig &config, const DataFieldEnum &df) noexcept
     return true;
 
   case DeviceConfig::PortType::RFCOMM:
+  case DeviceConfig::PortType::BLE_HM10:
+  case DeviceConfig::PortType::BLE_SENSOR:
     /* Bluetooth */
     if (new_type == config.port_type &&
         StringIsEqual(config.bluetooth_mac, df.GetAsString()))
@@ -848,6 +518,6 @@ DeviceEditWidget::OnModified(DataField &df) noexcept
       IsDataField(UseSecondDriver, df) || IsDataField(K6Bt, df))
     UpdateVisibilities();
 
-  if (listener != NULL)
+  if (listener != nullptr)
     listener->OnModified(*this);
 }
