@@ -25,6 +25,7 @@ Copyright_License {
 #include "DeviceEditWidget.hpp"
 #include "Vega/VegaDialogs.hpp"
 #include "BlueFly/BlueFlyDialogs.hpp"
+#include "ManageI2CPitotDialog.hpp"
 #include "ManageCAI302Dialog.hpp"
 #include "ManageFlarmDialog.hpp"
 #include "LX/ManageV7Dialog.hpp"
@@ -61,6 +62,7 @@ Copyright_License {
 
 #ifdef ANDROID
 #include "java/Global.hxx"
+#include "Android/Main.hpp"
 #include "Android/BluetoothHelper.hpp"
 #endif
 
@@ -76,7 +78,7 @@ class DeviceListWidget final
   struct Flags {
     bool duplicate:1;
     bool open:1, error:1;
-    bool alive:1, location:1, gps:1, baro:1, airspeed:1, vario:1, traffic:1;
+    bool alive:1, location:1, gps:1, baro:1, pitot:1, airspeed:1, vario:1, traffic:1;
     bool temperature:1;
     bool humidity:1;
     bool radio:1;
@@ -112,6 +114,7 @@ class DeviceListWidget final
       baro = basic.baro_altitude_available ||
         basic.pressure_altitude_available ||
         basic.static_pressure_available;
+      pitot = basic.pitot_pressure_available;
       airspeed = basic.airspeed_available;
       vario = basic.total_energy_vario_available;
       traffic = basic.flarm.IsDetected();
@@ -391,6 +394,11 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
       buffer.append(_("Baro"));
     }
 
+    if (flags.pitot) {
+      buffer.append(_T("; "));
+      buffer.append(_("Pitot"));
+    }
+
     if (flags.airspeed) {
       buffer.append(_T("; "));
       buffer.append(_("Airspeed"));
@@ -435,8 +443,10 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
     status = buffer;
 #ifdef ANDROID
   } else if ((config.port_type == DeviceConfig::PortType::RFCOMM ||
+              config.port_type == DeviceConfig::PortType::BLE_HM10 ||
               config.port_type == DeviceConfig::PortType::RFCOMM_SERVER) &&
-             !BluetoothHelper::isEnabled(Java::GetEnv())) {
+             bluetooth_helper != nullptr &&
+             !bluetooth_helper->IsEnabled(Java::GetEnv())) {
     status = _("Bluetooth is disabled");
 #endif
   } else if (flags.duplicate) {
@@ -503,8 +513,10 @@ DeviceListWidget::ReconnectCurrent()
   const DeviceConfig &config =
     CommonInterface::SetSystemSettings().devices[current];
   if ((config.port_type == DeviceConfig::PortType::RFCOMM ||
+       config.port_type == DeviceConfig::PortType::BLE_HM10 ||
        config.port_type == DeviceConfig::PortType::RFCOMM_SERVER) &&
-      !BluetoothHelper::isEnabled(Java::GetEnv())) {
+      bluetooth_helper != nullptr &&
+      !bluetooth_helper->IsEnabled(Java::GetEnv())) {
     ShowMessageBox(_("Bluetooth is disabled"), _("Reconnect"),
                    MB_OK | MB_ICONERROR);
     return;
@@ -597,6 +609,16 @@ DeviceListWidget::ManageCurrent()
   if (!descriptor.IsManageable())
     return;
 
+#ifdef ANDROID
+  const auto &config = descriptor.GetConfig();
+  if (config.port_type == DeviceConfig::PortType::DROIDSOAR_V2 ||
+      (config.port_type == DeviceConfig::PortType::I2CPRESSURESENSOR &&
+       config.press_use == DeviceConfig::PressureUse::PITOT)) {
+    ManageI2CPitotDialog(UIGlobals::GetMainWindow(), look, descriptor);
+    return;
+  }
+#endif
+
   if (descriptor.GetState() != PortState::READY) {
     ShowMessageBox(_("Device is not connected"), _("Manage"),
                    MB_OK | MB_ICONERROR);
