@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,40 +23,33 @@ Copyright_License {
 
 #include "ProfileListDialog.hpp"
 #include "ProfilePasswordDialog.hpp"
+#include "Dialogs/Error.hpp"
 #include "Dialogs/Message.hpp"
 #include "Dialogs/TextEntry.hpp"
 #include "Dialogs/WidgetDialog.hpp"
-#include "Widget/ListWidget.hpp"
-#include "Renderer/TextRowRenderer.hpp"
+#include "Widget/TextListWidget.hpp"
 #include "Form/Button.hpp"
-#include "Screen/Canvas.hpp"
-#include "Screen/Layout.hpp"
-#include "OS/FileUtil.hpp"
+#include "system/FileUtil.hpp"
+#include "system/Path.hpp"
 #include "LocalPath.hpp"
 #include "Profile/Map.hpp"
 #include "Profile/File.hpp"
 #include "UIGlobals.hpp"
-#include "Look/DialogLook.hpp"
 #include "Language/Language.hpp"
+#include "util/StaticString.hxx"
 
 #include <vector>
 
-#include <assert.h>
-#include <windef.h> /* for MAX_PATH */
-
-/* this macro exists in the WIN32 API */
-#ifdef DELETE
-#undef DELETE
-#endif
+#include <cassert>
 
 class ProfileListWidget final
-  : public ListWidget, private ActionListener {
+  : public TextListWidget {
 
   struct ListItem {
     StaticString<32> name;
-    StaticString<MAX_PATH> path;
+    AllocatedPath path;
 
-    ListItem(const TCHAR *_name, const TCHAR *_path)
+    ListItem(const TCHAR *_name, Path _path)
       :name(_name), path(_path) {}
 
     bool operator<(const ListItem &i2) const {
@@ -71,18 +64,11 @@ class ProfileListWidget final
   public:
     ProfileFileVisitor(std::vector<ListItem> &_list):list(_list) {}
 
-    void Visit(const TCHAR* path, const TCHAR* filename) {
-      list.emplace_back(filename, path);
+    void Visit(Path path, Path filename) override {
+      list.emplace_back(filename.c_str(), path);
     }
   };
 
-  enum Buttons {
-    NEW,
-    PASSWORD,
-    COPY,
-    DELETE,
-  };
-
   const bool select;
 
   WndForm *form;
@@ -91,28 +77,26 @@ class ProfileListWidget final
 
   std::vector<ListItem> list;
 
-  TextRowRenderer row_renderer;
-
 public:
-  ProfileListWidget(bool _select=false):select(_select) {}
+  ProfileListWidget(bool _select):select(_select) {}
 
   void CreateButtons(WidgetDialog &dialog);
 
   gcc_pure
-  tstring::const_pointer GetSelectedPath() const {
+  Path GetSelectedPath() const {
     if (list.empty())
       return nullptr;
 
     return list[GetList().GetCursorIndex()].path;
   }
 
-  void SelectPath(const TCHAR *path);
+  void SelectPath(Path path);
 
 private:
   void UpdateList();
 
   gcc_pure
-  int FindPath(const TCHAR *path) const;
+  int FindPath(Path path) const;
 
   void NewClicked();
   void PasswordClicked();
@@ -121,27 +105,23 @@ private:
 
 public:
   /* virtual methods from class Widget */
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override;
-  virtual void Unprepare() override;
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override;
 
 protected:
-  /* virtual methods from ListItemRenderer */
-  virtual void OnPaintItem(Canvas &canvas, const PixelRect rc,
-                           unsigned idx) override;
+  /* virtual methods from TextListWidget */
+  const TCHAR *GetRowText(unsigned i) const noexcept override {
+    return list[i].name;
+  }
 
   /* virtual methods from ListCursorHandler */
-  virtual bool CanActivateItem(unsigned index) const override {
+  bool CanActivateItem(unsigned index) const noexcept override {
     return select;
   }
 
-  virtual void OnActivateItem(unsigned index) override {
+  void OnActivateItem(unsigned index) noexcept override {
     form->SetModalResult(mrOK);
   }
-
-private:
-  /* virtual methods from class ActionListener */
-  virtual void OnAction(int id) override;
 };
 
 void
@@ -168,17 +148,17 @@ ProfileListWidget::UpdateList()
 }
 
 int
-ProfileListWidget::FindPath(const TCHAR *path) const
+ProfileListWidget::FindPath(Path path) const
 {
   for (unsigned n = list.size(), i = 0u; i < n; ++i)
-    if (StringIsEqual(path, list[i].path))
+    if (path == list[i].path)
       return i;
 
   return -1;
 }
 
 void
-ProfileListWidget::SelectPath(const TCHAR *path)
+ProfileListWidget::SelectPath(Path path)
 {
   auto i = FindPath(path);
   if (i >= 0)
@@ -190,35 +170,20 @@ ProfileListWidget::CreateButtons(WidgetDialog &dialog)
 {
   form = &dialog;
 
-  dialog.AddButton(_("New"), *this, NEW);
-  password_button = dialog.AddButton(_("Password"), *this, PASSWORD);
-  copy_button = dialog.AddButton(_("Copy"), *this, COPY);
-  delete_button = dialog.AddButton(_("Delete"), *this, DELETE);
+  dialog.AddButton(_("New"), [this](){ NewClicked(); });
+  password_button = dialog.AddButton(_("Password"), [this](){ PasswordClicked(); });
+  copy_button = dialog.AddButton(_("Copy"), [this](){ CopyClicked(); });
+  delete_button = dialog.AddButton(_("Delete"), [this](){ DeleteClicked(); });
 }
 
 void
-ProfileListWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
+ProfileListWidget::Prepare(ContainerWindow &parent,
+                           const PixelRect &rc) noexcept
 {
-  const DialogLook &look = UIGlobals::GetDialogLook();
-  CreateList(parent, look, rc,
-             row_renderer.CalculateLayout(*look.list.font));
+  TextListWidget::Prepare(parent, rc);
   UpdateList();
 }
 
-void
-ProfileListWidget::Unprepare()
-{
-  DeleteWindow();
-}
-
-void
-ProfileListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned i)
-{
-  assert(i < list.size());
-
-  row_renderer.DrawTextRow(canvas, rc, list[i].name);
-}
-
 inline void
 ProfileListWidget::NewClicked()
 {
@@ -231,9 +196,7 @@ ProfileListWidget::NewClicked()
   filename = name;
   filename += _T(".prf");
 
-  StaticString<MAX_PATH> buffer;
-  const auto path = LocalPath(buffer.buffer(), filename);
-
+  const auto path = LocalPath(filename);
   if (!File::CreateExclusive(path)) {
     ShowMessageBox(name, _("File exists already."), MB_OK|MB_ICONEXCLAMATION);
     return;
@@ -251,9 +214,11 @@ ProfileListWidget::PasswordClicked()
   const auto &item = list[GetList().GetCursorIndex()];
 
   ProfileMap data;
-  if (!Profile::LoadFile(data, item.path)) {
-    ShowMessageBox(item.name, _("Failed to load file."),
-                   MB_OK|MB_ICONEXCLAMATION);
+
+  try {
+    Profile::LoadFile(data, item.path);
+  } catch (...) {
+    ShowError(std::current_exception(), _("Failed to load file."));
     return;
   }
 
@@ -261,9 +226,12 @@ ProfileListWidget::PasswordClicked()
       !SetProfilePasswordDialog(data))
     return;
 
-  if (!Profile::SaveFile(data, item.path))
-    ShowMessageBox(item.name, _("Failed to save file."),
-                   MB_OK|MB_ICONEXCLAMATION);
+  try {
+    Profile::SaveFile(data, item.path);
+  } catch (...) {
+    ShowError(std::current_exception(), _("Failed to save file."));
+    return;
+  }
 }
 
 inline void
@@ -272,13 +240,14 @@ ProfileListWidget::CopyClicked()
   assert(GetList().GetCursorIndex() < list.size());
 
   const auto &item = list[GetList().GetCursorIndex()];
-  const TCHAR *old_path = item.path;
-  const TCHAR *old_filename = item.name;
+  const Path old_path = item.path;
 
   ProfileMap data;
-  if (!Profile::LoadFile(data, old_path)) {
-    ShowMessageBox(old_filename, _("Failed to load file."),
-                   MB_OK|MB_ICONEXCLAMATION);
+
+  try {
+    Profile::LoadFile(data, old_path);
+  } catch (...) {
+    ShowError(std::current_exception(), _("Failed to load file."));
     return;
   }
 
@@ -294,8 +263,7 @@ ProfileListWidget::CopyClicked()
   new_filename = new_name;
   new_filename += _T(".prf");
 
-  StaticString<MAX_PATH> buffer;
-  const auto new_path = LocalPath(buffer.buffer(), new_filename);
+  const auto new_path = LocalPath(new_filename);
 
   if (File::ExistsAny(new_path)) {
     ShowMessageBox(new_name, _("File exists already."),
@@ -303,9 +271,10 @@ ProfileListWidget::CopyClicked()
     return;
   }
 
-  if (!Profile::SaveFile(data, new_path)) {
-    ShowMessageBox(new_name, _("Failed to save file."),
-                   MB_OK|MB_ICONEXCLAMATION);
+  try {
+    Profile::SaveFile(data, new_path);
+  } catch (...) {
+    ShowError(std::current_exception(), _("Failed to save file."));
     return;
   }
 
@@ -333,20 +302,25 @@ ProfileListWidget::DeleteClicked()
 
   const auto &item = list[GetList().GetCursorIndex()];
 
-  const auto password_result = CheckProfileFilePassword(item.path);
-  switch (password_result) {
-  case ProfilePasswordResult::UNPROTECTED:
-    if (!ConfirmDeleteProfile(item.name))
+  try {
+    const auto password_result = CheckProfileFilePassword(item.path);
+    switch (password_result) {
+    case ProfilePasswordResult::UNPROTECTED:
+      if (!ConfirmDeleteProfile(item.name))
+        return;
+
+      break;
+
+    case ProfilePasswordResult::MATCH:
+      break;
+
+    case ProfilePasswordResult::MISMATCH:
+    case ProfilePasswordResult::CANCEL:
+      CheckProfilePasswordResult(password_result);
       return;
-
-    break;
-
-  case ProfilePasswordResult::MATCH:
-    break;
-
-  case ProfilePasswordResult::MISMATCH:
-  case ProfilePasswordResult::CANCEL:
-    CheckProfilePasswordResult(password_result);
+    }
+  } catch (...) {
+    ShowError(std::current_exception(), _("Password"));
     return;
   }
 
@@ -354,67 +328,40 @@ ProfileListWidget::DeleteClicked()
   UpdateList();
 }
 
-void
-ProfileListWidget::OnAction(int id)
-{
-  switch ((Buttons)id) {
-  case NEW:
-    NewClicked();
-    break;
-
-  case PASSWORD:
-    PasswordClicked();
-    break;
-
-  case COPY:
-    CopyClicked();
-    break;
-
-  case DELETE:
-    DeleteClicked();
-    break;
-  }
-}
-
 void
 ProfileListDialog()
 {
-  ProfileListWidget widget;
-  WidgetDialog dialog(UIGlobals::GetDialogLook());
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("Profiles"), &widget);
-  widget.CreateButtons(dialog);
+  TWidgetDialog<ProfileListWidget>
+    dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+           UIGlobals::GetDialogLook(), _("Profiles"));
   dialog.AddButton(_("Close"), mrOK);
+  dialog.SetWidget(false);
+  dialog.GetWidget().CreateButtons(dialog);
   dialog.EnableCursorSelection();
 
   dialog.ShowModal();
-  dialog.StealWidget();
 }
 
-tstring
-SelectProfileDialog(tstring::const_pointer selected_path)
+AllocatedPath
+SelectProfileDialog(Path selected_path)
 {
-  ProfileListWidget widget(true);
-  WidgetDialog dialog(UIGlobals::GetDialogLook());
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("Select profile"), &widget);
+  TWidgetDialog<ProfileListWidget>
+    dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+           UIGlobals::GetDialogLook(), _("Select profile"));
+  dialog.SetWidget(true);
   dialog.AddButton(_("Select"), mrOK);
-  widget.CreateButtons(dialog);
+  dialog.GetWidget().CreateButtons(dialog);
   dialog.AddButton(_("Cancel"), mrCancel);
   dialog.EnableCursorSelection();
 
-  if (selected_path != nullptr) {
+  if (!selected_path.IsNull()) {
     dialog.PrepareWidget();
-    widget.SelectPath(selected_path);
+    dialog.GetWidget().SelectPath(selected_path);
   }
 
   auto result = dialog.ShowModal();
 
-  selected_path = result == mrOK
-    ? widget.GetSelectedPath()
+  return result == mrOK
+    ? dialog.GetWidget().GetSelectedPath()
     : nullptr;
-  dialog.StealWidget();
-
-  if (selected_path == nullptr)
-    return tstring();
-
-  return selected_path;
 }
