@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -33,6 +33,7 @@ Copyright_License {
 
 enum ControlIndex {
   PilotName,
+  CoPilotName,
   LoggerTimeStepCruise,
   LoggerTimeStepCircling,
   DisableAutoLogger,
@@ -47,8 +48,8 @@ public:
     :RowFormWidget(UIGlobals::GetDialogLook()) {}
 
 public:
-  virtual void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
-  virtual bool Save(bool &changed) override;
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
+  bool Save(bool &changed) noexcept override;
 };
 
 static constexpr StaticEnumChoice auto_logger_list[] = {
@@ -59,7 +60,8 @@ static constexpr StaticEnumChoice auto_logger_list[] = {
 };
 
 void
-LoggerConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
+LoggerConfigPanel::Prepare(ContainerWindow &parent,
+                           const PixelRect &rc) noexcept
 {
   const ComputerSettings &settings_computer = CommonInterface::GetComputerSettings();
   const LoggerSettings &logger = settings_computer.logger;
@@ -67,6 +69,8 @@ LoggerConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   RowFormWidget::Prepare(parent, rc);
   AddText(_("Pilot name"), nullptr, logger.pilot_name);
 
+  AddText(_("CoPilot name"), nullptr, logger.copilot_name);
+
   AddTime(_("Time step cruise"),
           _("This is the time interval between logged points when not circling."),
           1, 30, 1, logger.time_step_cruise);
@@ -98,7 +102,7 @@ LoggerConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
 }
 
 bool
-LoggerConfigPanel::Save(bool &changed)
+LoggerConfigPanel::Save(bool &changed) noexcept
 {
   ComputerSettings &settings_computer = CommonInterface::SetComputerSettings();
   LoggerSettings &logger = settings_computer.logger;
@@ -106,6 +110,9 @@ LoggerConfigPanel::Save(bool &changed)
   changed |= SaveValue(PilotName, ProfileKeys::PilotName,
                        logger.pilot_name);
 
+  changed |= SaveValue(CoPilotName, ProfileKeys::CoPilotName,
+                       logger.copilot_name);
+
   changed |= SaveValue(LoggerTimeStepCruise, ProfileKeys::LoggerTimeStepCruise,
                        logger.time_step_cruise);
 
@@ -137,8 +144,8 @@ LoggerConfigPanel::Save(bool &changed)
   return true;
 }
 
-Widget *
+std::unique_ptr<Widget>
 CreateLoggerConfigPanel()
 {
-  return new LoggerConfigPanel();
+  return std::make_unique<LoggerConfigPanel>();
 }
