@@ -220,12 +220,13 @@ dlgStartupShowModal()
 
   /* preselect the most recently used profile */
   unsigned best_index = 0;
-  uint64_t best_timestamp = 0;
+  std::chrono::system_clock::time_point best_timestamp =
+    std::chrono::system_clock::time_point::min();
   unsigned length = dff->size();
 
   for (unsigned i = 0; i < length; ++i) {
     const auto path = dff->GetItem(i);
-    uint64_t timestamp = File::GetLastModification(path);
+    const auto timestamp = File::GetLastModification(path);
     if (timestamp > best_timestamp) {
       best_timestamp = timestamp;
       best_index = i;
