@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,21 +25,22 @@ Copyright_License {
 #define XCSOAR_WAYPOINT_INFO_WIDGET_HPP
 
 #include "Widget/RowFormWidget.hpp"
+#include "Engine/Waypoint/Ptr.hpp"
 
 struct Waypoint;
 struct GlideResult;
 
 struct WaypointInfoWidget : public RowFormWidget {
-  const Waypoint &waypoint;
+  const WaypointPtr waypoint;
 
 public:
-  WaypointInfoWidget(const DialogLook &look, const Waypoint &_waypoint)
-    :RowFormWidget(look), waypoint(_waypoint) {}
+  WaypointInfoWidget(const DialogLook &look, WaypointPtr _waypoint) noexcept
+    :RowFormWidget(look), waypoint(std::move(_waypoint)) {}
 
-  void AddGlideResult(const TCHAR *label, const GlideResult &result);
+  void AddGlideResult(const TCHAR *label, const GlideResult &result) noexcept;
 
   /* methods from Widget */
-  void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
 };
 
 #endif
