@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,9 +24,9 @@ Copyright_License {
 #ifndef XCSOAR_LEGAL_POINT_SET_HPP
 #define XCSOAR_LEGAL_POINT_SET_HPP
 
-#include "Util/EnumBitSet.hpp"
+#include "util/EnumBitSet.hxx"
 #include "TaskPointFactoryType.hpp"
 
-typedef EnumBitSet<TaskPointFactoryType> LegalPointSet;
+using LegalPointSet = EnumBitSet<TaskPointFactoryType>;
 
 #endif
