@@ -2,7 +2,7 @@
   Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -33,8 +33,15 @@ public:
   /**
    * Generate boundary points for the arc described by the parameters.
    * This excludes the points at the start/end angle.
+   *
+   * The arc is generated in an increasing angle order - i.e. CW.
+   *
+   * @param center The origin of the radials - phi, lambda
+   * @param radius The radius of the arc - meters.
+   * @param start_radial The most CCW portion of the arc - radians.
+   * @param end_radial The most CW portion of the arc - radians.
    */
-  void GenerateArcExcluding(const GeoPoint &center, fixed radius,
+  void GenerateArcExcluding(const GeoPoint &center, double radius,
                             Angle start_radial, Angle end_radial);
 };
 
