@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -26,10 +26,12 @@
 #include "Geo/Flat/TaskProjection.hpp"
 #include "Task/AbstractTask.hpp"
 #include "SmartTaskAdvance.hpp"
-#include "Util/DereferenceIterator.hpp"
-#include "Util/StaticString.hxx"
+#include "Waypoint/Ptr.hpp"
+#include "util/DereferenceIterator.hxx"
+#include "util/StaticString.hxx"
 
-#include <assert.h>
+#include <cassert>
+#include <memory>
 #include <vector>
 
 class SearchPoint;
@@ -40,10 +42,9 @@ class FinishPoint;
 class AbstractTaskFactory;
 class TaskDijkstraMin;
 class TaskDijkstraMax;
-struct Waypoint;
 class Waypoints;
 class AATPoint;
-class FlatBoundingBox;
+struct FlatBoundingBox;
 class GeoBounds;
 struct TaskSummary;
 struct TaskFactoryConstraints;
@@ -63,28 +64,33 @@ class OrderedTask final : public AbstractTask
 {
 public:
   /** Storage type of task points */
-  typedef std::vector<OrderedTaskPoint*> OrderedTaskPointVector;
+  using OrderedTaskPointVector = std::vector<std::unique_ptr<OrderedTaskPoint>>;
 
-  typedef DereferenceContainerAdapter<const OrderedTaskPointVector,
-                                      const OrderedTaskPoint> ConstTaskPointList;
+  using ConstTaskPointList =
+    DereferenceContainerAdapter<const OrderedTaskPointVector,
+                                const OrderedTaskPoint>;
+
+  using TaskPointList =
+    DereferenceContainerAdapter<const OrderedTaskPointVector,
+                                OrderedTaskPoint>;
 
 private:
   OrderedTaskPointVector task_points;
   OrderedTaskPointVector optional_start_points;
 
-  StartPoint *taskpoint_start;
-  FinishPoint *taskpoint_finish;
+  StartPoint *taskpoint_start = nullptr;
+  FinishPoint *taskpoint_finish = nullptr;
 
   TaskProjection task_projection;
 
   GeoPoint last_min_location;
 
   TaskFactoryType factory_mode;
-  AbstractTaskFactory* active_factory;
+  std::unique_ptr<AbstractTaskFactory> active_factory;
   OrderedTaskSettings ordered_settings;
   SmartTaskAdvance task_advance;
-  TaskDijkstraMin *dijkstra_min;
-  TaskDijkstraMax *dijkstra_max;
+  std::unique_ptr<TaskDijkstraMin> dijkstra_min;
+  std::unique_ptr<TaskDijkstraMax> dijkstra_max;
 
   StaticString<64> name;
 
@@ -99,20 +105,20 @@ public:
    *
    * @return Initialised object
    */
-  explicit OrderedTask(const TaskBehaviour &tb);
-  ~OrderedTask();
+  explicit OrderedTask(const TaskBehaviour &tb) noexcept;
+  ~OrderedTask() noexcept;
 
   /**
    * Accessor for factory system for constructing tasks
    *
    * @return Factory
    */
-  gcc_pure
-  AbstractTaskFactory& GetFactory() const {
+  [[gnu::pure]]
+  AbstractTaskFactory &GetFactory() const noexcept {
     return *active_factory;
   }
 
-  gcc_pure
+  [[gnu::pure]]
   const TaskFactoryConstraints &GetFactoryConstraints() const;
 
   /**
@@ -129,7 +135,7 @@ public:
    *
    * @return Vector of factory types
    */
-  gcc_pure
+  [[gnu::pure]]
   std::vector<TaskFactoryType> GetFactoryTypes(bool all = true) const;
 
   void SetTaskBehaviour(const TaskBehaviour &tb);
@@ -154,8 +160,7 @@ public:
    *
    * @return Initialised object
    */
-  gcc_malloc
-  OrderedTask *Clone(const TaskBehaviour &tb) const;
+  std::unique_ptr<OrderedTask> Clone(const TaskBehaviour &tb) const noexcept;
 
   /**
    * Copy task into this task
@@ -171,7 +176,7 @@ public:
    *
    * @return Index of active task point sequence
    */
-  gcc_pure
+  [[gnu::pure]]
   unsigned GetActiveIndex() const {
     return active_task_point;
   }
@@ -183,7 +188,7 @@ public:
    *
    * @return OrderedTaskPoint at index
    */
-  gcc_pure
+  [[gnu::pure]]
   const OrderedTaskPoint &GetTaskPoint(const unsigned index) const {
     assert(index < task_points.size());
 
@@ -195,7 +200,7 @@ public:
    *
    * @return True if task has start
    */
-  gcc_pure
+  [[gnu::pure]]
   bool HasStart() const {
     return taskpoint_start != nullptr;
   }
@@ -205,7 +210,7 @@ public:
    *
    * @return True if task has finish
    */
-  gcc_pure
+  [[gnu::pure]]
   bool HasFinish() const {
     return taskpoint_finish != nullptr;
   }
@@ -219,7 +224,7 @@ public:
   /**
    * Returns true if there are optional start points.
    */
-  gcc_pure
+  [[gnu::pure]]
   bool HasOptionalStarts() const {
     return !optional_start_points.empty();
   }
@@ -311,7 +316,7 @@ public:
    * @param waypoint
    * @return true if succeeded
    */
-  bool RelocateOptionalStart(const unsigned position, const Waypoint& waypoint);
+  bool RelocateOptionalStart(const unsigned position, WaypointPtr &&waypoint);
 
   /**
    * Relocate a task point to a new location
@@ -321,7 +326,7 @@ public:
    *
    * @return True on success
    */
-  bool Relocate(const unsigned position, const Waypoint& waypoint);
+  bool Relocate(const unsigned position, WaypointPtr &&waypoint);
 
  /**
   * returns pointer to AATPoint accessed via TPIndex if exist
@@ -335,7 +340,7 @@ public:
   /**
    * Check whether the task point with the specified index exists.
    */
-  gcc_pure
+  [[gnu::pure]]
   bool IsValidIndex(unsigned i) const {
     return i < task_points.size();
   }
@@ -345,17 +350,21 @@ public:
    *
    * @return True if task is full
    */
-  gcc_pure
+  [[gnu::pure]]
   bool IsFull() const;
 
   /**
    * Accessor for task projection, for use when creating task points
    *
+   * This object is only valid if the task is not empty (see IsEmpty()).
+   *
    * @return Task global projection
    */
-  gcc_pure
+  [[gnu::pure]]
   const TaskProjection&
   GetTaskProjection() const {
+    assert(!IsEmpty());
+
     return task_projection;
   }
 
@@ -376,13 +385,6 @@ public:
    */
   void UpdateGeometry();
 
-  /**
-   * Convert a GeoBounds into a flat bounding box projected
-   * according to the task projection.
-   */
-  gcc_pure
-  FlatBoundingBox GetBoundingBox(const GeoBounds &bounds) const;
-
   /**
    * Update summary task statistics (progress along path)
    */
@@ -440,14 +442,14 @@ private:
   bool RunDijsktraMin(const GeoPoint &location);
 
 
-  fixed ScanDistanceMin(const GeoPoint &ref, bool full);
+  double ScanDistanceMin(const GeoPoint &ref, bool full);
 
   /**
    * @return true if a solution was found (and applied)
    */
   bool RunDijsktraMax();
 
-  fixed ScanDistanceMax();
+  double ScanDistanceMax();
 
   /**
    * Optimise target ranges (for adjustable tasks) to produce an estimated
@@ -458,9 +460,9 @@ private:
    *
    * @return Target range parameter (0-1)
    */
-  fixed CalcMinTarget(const AircraftState &state_now,
-                      const GlidePolar &glide_polar,
-                      const fixed t_target);
+  double CalcMinTarget(const AircraftState &state_now,
+                       const GlidePolar &glide_polar,
+                       const double t_target);
 
   /**
    * Sets previous/next taskpoint pointers for task point at specified
@@ -487,11 +489,11 @@ private:
   void UpdateStartTransition(const AircraftState &state,
                              OrderedTaskPoint &start);
 
-  gcc_pure
+  [[gnu::pure]]
   bool DistanceIsSignificant(const GeoPoint &location,
                              const GeoPoint &location_last) const;
 
-  gcc_pure
+  [[gnu::pure]]
   bool AllowIncrementalBoundaryStats(const AircraftState &state) const;
 
   bool CheckTransitionPoint(OrderedTaskPoint &point,
@@ -574,11 +576,21 @@ protected:
   void PropagateOrderedTaskSettings();
 
 public:
+  /**
+   * Check whether the task is empty.
+   *
+   * Note that even if the task is non-empty, it does not imply that
+   * the task is "valid".
+   */
+  bool IsEmpty() const noexcept {
+    return task_points.empty();
+  }
+
   ConstTaskPointList GetPoints() const {
     return task_points;
   }
 
-  gcc_pure
+  [[gnu::pure]]
   OrderedTaskPoint &GetPoint(const unsigned i) {
     assert(i < task_points.size());
     assert(task_points[i] != nullptr);
@@ -586,7 +598,7 @@ public:
     return *task_points[i];
   }
 
-  gcc_pure
+  [[gnu::pure]]
   const OrderedTaskPoint &GetPoint(const unsigned i) const {
     assert(i < task_points.size());
     assert(task_points[i] != nullptr);
@@ -601,7 +613,7 @@ public:
   /**
    * @return number of optional start poitns
    */
-  gcc_pure
+  [[gnu::pure]]
   unsigned GetOptionalStartPointCount() const {
     return optional_start_points.size();
   }
@@ -612,7 +624,7 @@ public:
    * @param pos optional start point index
    * @return nullptr if index out of range, else optional start point
    */
-  gcc_pure
+  [[gnu::pure]]
   const OrderedTaskPoint &GetOptionalStartPoint(unsigned i) const {
     assert(i < optional_start_points.size());
 
@@ -620,7 +632,7 @@ public:
   }
 
   /** Determines whether the task has adjustable targets */
-  gcc_pure
+  [[gnu::pure]]
   bool HasTargets() const;
 
   /**
@@ -628,16 +640,22 @@ public:
    *
    * @return Location of center of task or GeoPoint::Invalid()
    */
-  gcc_pure
-  GeoPoint GetTaskCenter() const;
+  [[gnu::pure]]
+  GeoPoint GetTaskCenter() const noexcept {
+    assert(!IsEmpty());
+    return task_projection.GetCenter();
+  }
 
   /**
    * Find approximate radius of task from center to edge (for rendering purposes)
    *
    * @return Radius (m) from center to edge of task
    */
-  gcc_pure
-  fixed GetTaskRadius() const;
+  [[gnu::pure]]
+  double GetTaskRadius() const noexcept {
+    assert(!IsEmpty());
+    return task_projection.ApproxRadius();
+  }
 
   /**
    * returns the index of the highest intermediate TP that has been entered.
@@ -648,7 +666,7 @@ public:
    */
   unsigned GetLastIntermediateAchieved() const;
 
-  gcc_pure
+  [[gnu::pure]]
   const StaticString<64> &GetName() const {
     return name;
   }
@@ -663,55 +681,55 @@ public:
 
 public:
   /* virtual methods from class TaskInterface */
-  unsigned TaskSize() const override {
+  unsigned TaskSize() const noexcept override {
     return task_points.size();
   }
 
-  void SetActiveTaskPoint(unsigned desired) override;
-  TaskWaypoint *GetActiveTaskPoint() const override;
-  bool IsValidTaskPoint(const int index_offset=0) const override;
+  void SetActiveTaskPoint(unsigned desired) noexcept override;
+  TaskWaypoint *GetActiveTaskPoint() const noexcept override;
+  bool IsValidTaskPoint(const int index_offset=0) const noexcept override;
   bool UpdateIdle(const AircraftState& state_now,
-                  const GlidePolar &glide_polar) override;
+                  const GlidePolar &glide_polar) noexcept override;
 
   /* virtual methods from class AbstractTask */
-  void Reset() override;
-  bool TaskStarted(bool soft=false) const override;
-  bool CheckTask() const override;
+  void Reset() noexcept override;
+  bool TaskStarted(bool soft=false) const noexcept override;
+  TaskValidationErrorSet CheckTask() const noexcept override;
 
 protected:
   /* virtual methods from class AbstractTask */
   bool UpdateSample(const AircraftState &state_now,
                     const GlidePolar &glide_polar,
-                    const bool full_update) override;
+                    const bool full_update) noexcept override;
   bool CheckTransitions(const AircraftState &state_now,
-                        const AircraftState &state_last) override;
+                        const AircraftState &state_last) noexcept override;
   bool CalcBestMC(const AircraftState &state_now,
                   const GlidePolar &glide_polar,
-                  fixed& best) const override;
-  fixed CalcRequiredGlide(const AircraftState &state_now,
-                          const GlidePolar &glide_polar) const override;
+                  double &best) const noexcept override;
+  double CalcRequiredGlide(const AircraftState &state_now,
+                           const GlidePolar &glide_polar) const noexcept override;
   bool CalcCruiseEfficiency(const AircraftState &state_now,
                             const GlidePolar &glide_polar,
-                            fixed &value) const override;
+                            double &value) const noexcept override;
   bool CalcEffectiveMC(const AircraftState &state_now,
                        const GlidePolar &glide_polar,
-                       fixed &value) const override;
-  fixed CalcGradient(const AircraftState &state_now) const override;
-  fixed ScanTotalStartTime() override;
-  fixed ScanLegStartTime() override;
-  fixed ScanDistanceNominal() override;
-  fixed ScanDistancePlanned() override;
-  fixed ScanDistanceRemaining(const GeoPoint &ref) override;
-  fixed ScanDistanceScored(const GeoPoint &ref) override;
-  fixed ScanDistanceTravelled(const GeoPoint &ref) override;
+                       double &value) const noexcept override;
+  double CalcGradient(const AircraftState &state_now) const noexcept override;
+  double ScanTotalStartTime() noexcept override;
+  double ScanLegStartTime() noexcept override;
+  double ScanDistanceNominal() noexcept override;
+  double ScanDistancePlanned() noexcept override;
+  double ScanDistanceRemaining(const GeoPoint &ref) noexcept override;
+  double ScanDistanceScored(const GeoPoint &ref) noexcept override;
+  double ScanDistanceTravelled(const GeoPoint &ref) noexcept override;
   void ScanDistanceMinMax(const GeoPoint &ref, bool full,
-                          fixed *dmin, fixed *dmax) override;
+                          double *dmin, double *dmax) noexcept override;
   void GlideSolutionRemaining(const AircraftState &state_now,
                               const GlidePolar &polar,
-                              GlideResult &total, GlideResult &leg) override;
+                              GlideResult &total, GlideResult &leg) noexcept override;
   void GlideSolutionTravelled(const AircraftState &state_now,
                               const GlidePolar &glide_polar,
-                              GlideResult &total, GlideResult &leg) override;
+                              GlideResult &total, GlideResult &leg) noexcept override;
   void GlideSolutionPlanned(const AircraftState &state_now,
                             const GlidePolar &glide_polar,
                             GlideResult &total,
@@ -719,9 +737,9 @@ protected:
                             DistanceStat &total_remaining_effective,
                             DistanceStat &leg_remaining_effective,
                             const GlideResult &solution_remaining_total,
-                            const GlideResult &solution_remaining_leg) override;
+                            const GlideResult &solution_remaining_leg) noexcept override;
 protected:
-  bool IsScored() const override;
+  bool IsScored() const noexcept override;
 
 public:
   void AcceptTaskPointVisitor(TaskPointConstVisitor &visitor) const override;
