@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,10 +23,10 @@ Copyright_License {
 
 #include "GeoPoint.hpp"
 #include "Formatter/GeoPointFormatter.hpp"
-#include "Util/Macros.hpp"
+#include "util/Macros.hpp"
 
 void
-GeoPointDataField::ModifyValue(GeoPoint _value)
+GeoPointDataField::ModifyValue(GeoPoint _value) noexcept
 {
   if (_value == value)
     return;
@@ -36,7 +36,7 @@ GeoPointDataField::ModifyValue(GeoPoint _value)
 }
 
 const TCHAR *
-GeoPointDataField::GetAsString() const
+GeoPointDataField::GetAsString() const noexcept
 {
   if (!value.IsValid())
     return _T("");
