@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -28,7 +28,7 @@ class DataField;
 
 class DataFieldListener {
 public:
-  virtual void OnModified(DataField &df) = 0;
+  virtual void OnModified(DataField &df) noexcept = 0;
 };
 
 #endif
