@@ -24,8 +24,8 @@ Copyright_License {
 #ifndef XCSOAR_TIME_FORMATTER_HPP
 #define XCSOAR_TIME_FORMATTER_HPP
 
+#include "time/FloatDuration.hxx"
 #include "util/StringBuffer.hxx"
-#include "util/Compiler.h"
 
 #include <tchar.h>
 
@@ -55,10 +55,22 @@ FormatISO8601(TCHAR *buffer, const BrokenDateTime &stamp) noexcept;
 #endif
 
 void
-FormatTime(TCHAR *buffer, double time) noexcept;
+FormatTime(TCHAR *buffer, FloatDuration time) noexcept;
+
+static inline void
+FormatTime(TCHAR *buffer, double time) noexcept
+{
+  FormatTime(buffer, FloatDuration{time});
+}
 
 void
-FormatTimeLong(TCHAR *buffer, double time) noexcept;
+FormatTimeLong(TCHAR *buffer, FloatDuration time) noexcept;
+
+static inline void
+FormatTimeLong(TCHAR *buffer, double time) noexcept
+{
+  FormatTimeLong(buffer, FloatDuration{time});
+}
 
 /**
  * precedes with "-" if time is negative
@@ -66,17 +78,24 @@ FormatTimeLong(TCHAR *buffer, double time) noexcept;
  * @param time input seconds
  */
 void
-FormatSignedTimeHHMM(TCHAR* buffer, int time) noexcept;
+FormatSignedTimeHHMM(TCHAR *buffer, std::chrono::seconds time) noexcept;
 
-gcc_const
+[[gnu::const]]
 static inline BasicStringBuffer<TCHAR, 8>
-FormatSignedTimeHHMM(int time) noexcept
+FormatSignedTimeHHMM(std::chrono::seconds time) noexcept
 {
   BasicStringBuffer<TCHAR, 8> buffer;
   FormatSignedTimeHHMM(buffer.data(), time);
   return buffer;
 }
 
+[[gnu::const]]
+static inline BasicStringBuffer<TCHAR, 8>
+FormatSignedTimeHHMM(int time) noexcept
+{
+  return FormatSignedTimeHHMM(std::chrono::seconds{time});
+}
+
 /**
  * sets HHMMSSSmart and SSSmart
  * if hours > 0, returns HHMM in buffer1 and SS in buffer2
@@ -84,16 +103,24 @@ FormatSignedTimeHHMM(int time) noexcept
  * @param d input seconds
  */
 void
-FormatTimeTwoLines(TCHAR *buffer1, TCHAR *buffer2, int time) noexcept;
+FormatTimeTwoLines(TCHAR *buffer1, TCHAR *buffer2,
+                   std::chrono::seconds time) noexcept;
+
+static inline void
+FormatTimeTwoLines(TCHAR *buffer1, TCHAR *buffer2,
+                   int time) noexcept
+{
+  FormatTimeTwoLines(buffer1, buffer2, std::chrono::seconds{time});
+}
 
 void
-FormatTimespanSmart(TCHAR *buffer, int timespan,
+FormatTimespanSmart(TCHAR *buffer, std::chrono::seconds timespan,
                     unsigned max_tokens = 1,
                     const TCHAR *separator = _T(" ")) noexcept;
 
-gcc_const
+[[gnu::const]]
 static inline BasicStringBuffer<TCHAR, 64>
-FormatTimespanSmart(int timespan, unsigned max_tokens = 1,
+FormatTimespanSmart(std::chrono::seconds timespan, unsigned max_tokens = 1,
                     const TCHAR *separator = _T(" ")) noexcept
 {
   BasicStringBuffer<TCHAR, 64> buffer;
@@ -101,4 +128,13 @@ FormatTimespanSmart(int timespan, unsigned max_tokens = 1,
   return buffer;
 }
 
+[[gnu::const]]
+static inline auto
+FormatTimespanSmart(int timespan, unsigned max_tokens = 1,
+                    const TCHAR *separator = _T(" ")) noexcept
+{
+  return FormatTimespanSmart(std::chrono::seconds{timespan},
+                             max_tokens, separator);
+}
+
 #endif
