@@ -75,15 +75,8 @@ LoggerImpl::PreTakeoffBuffer::operator=(const NMEAInfo &src)
   return *this;
 }
 
-LoggerImpl::LoggerImpl()
-  :filename(nullptr), writer(nullptr)
-{
-}
-
-LoggerImpl::~LoggerImpl()
-{
-  delete writer;
-}
+LoggerImpl::LoggerImpl() = default;
+LoggerImpl::~LoggerImpl() noexcept = default;
 
 void
 LoggerImpl::StopLogger(const NMEAInfo &gps_info)
@@ -102,14 +95,13 @@ LoggerImpl::StopLogger(const NMEAInfo &gps_info)
   LogFormat(_T("Logger stopped: %s"), filename.c_str());
 
   // Logger off
-  delete writer;
-  writer = nullptr;
+  writer.reset();
 
   pre_takeoff_buffer.clear();
 }
 
 void
-LoggerImpl::LogPointToBuffer(const NMEAInfo &gps_info)
+LoggerImpl::LogPointToBuffer(const NMEAInfo &gps_info) noexcept
 {
   assert(gps_info.alive);
   assert(gps_info.time_available);
@@ -233,7 +225,7 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
   const auto logs_path = MakeLocalPath(_T("logs"));
 
   const BrokenDate today = gps_info.date_time_utc.IsDatePlausible()
-    ? (const BrokenDate &)gps_info.date_time_utc
+    ? gps_info.date_time_utc.GetDate()
     : BrokenDate::TodayUTC();
 
   StaticString<64> name;
@@ -248,7 +240,7 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
   frecord.Reset();
 
   try {
-    writer = new IGCWriter(filename);
+    writer = std::make_unique<IGCWriter>(filename);
   } catch (...) {
     LogError(std::current_exception());
     return false;
@@ -265,8 +257,9 @@ LoggerImpl::LoggerNote(const TCHAR *text)
     writer->LoggerNote(text);
 }
 
+[[gnu::pure]]
 static const TCHAR *
-GetGPSDeviceName()
+GetGPSDeviceName() noexcept
 {
   if (is_simulator())
     return _T("Simulator");
@@ -321,7 +314,7 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
 }
 
 void
-LoggerImpl::ClearBuffer()
+LoggerImpl::ClearBuffer() noexcept
 {
   pre_takeoff_buffer.clear();
 }
