@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,24 +23,25 @@ Copyright_License {
 
 #include "GlueMapWindow.hpp"
 #include "Look/MapLook.hpp"
-#include "Screen/Icon.hpp"
+#include "ui/canvas/Icon.hpp"
 #include "Language/Language.hpp"
 #include "Screen/Layout.hpp"
 #include "Task/ProtectedTaskManager.hpp"
+#include "Engine/Task/TaskManager.hpp"
 #include "Engine/Task/Ordered/OrderedTask.hpp"
 #include "Renderer/TextInBox.hpp"
-#include "Terrain/RasterWeatherCache.hpp"
-#include "Terrain/RasterWeatherStore.hpp"
+#include "Weather/Rasp/RaspRenderer.hpp"
 #include "Formatter/UserUnits.hpp"
 #include "Formatter/UserGeoPointFormatter.hpp"
 #include "UIState.hpp"
 #include "Renderer/FinalGlideBarRenderer.hpp"
 #include "Terrain/RasterTerrain.hpp"
-#include "Util/Macros.hpp"
-#include "Util/Clamp.hpp"
-#include "Util/StringAPI.hxx"
+#include "util/Macros.hpp"
+#include "util/Clamp.hpp"
+#include "util/StringAPI.hxx"
 #include "Look/GestureLook.hpp"
 #include "Input/InputEvents.hpp"
+#include "Renderer/MapScaleRenderer.hpp"
 
 #include <stdio.h>
 
@@ -71,15 +72,12 @@ GlueMapWindow::DrawCrossHairs(Canvas &canvas) const
   if (!render_projection.IsValid())
     return;
 
-  Pen dash_pen(Pen::DASH, 1, COLOR_DARK_GRAY);
-  canvas.Select(dash_pen);
+  canvas.Select(look.overlay.crosshair_pen);
 
-  const RasterPoint center = render_projection.GetScreenOrigin();
+  const auto center = render_projection.GetScreenOrigin();
 
-  canvas.DrawLine(center.x + 20, center.y,
-              center.x - 20, center.y);
-  canvas.DrawLine(center.x, center.y + 20,
-              center.x, center.y - 20);
+  canvas.DrawLine(center.At(20, 0), center.At(-20, 0));
+  canvas.DrawLine(center.At(0, 20), center.At(0, -20));
 }
 
 void
@@ -94,31 +92,29 @@ GlueMapWindow::DrawPanInfo(Canvas &canvas) const
   mode.shape = LabelShape::OUTLINED;
   mode.align = TextInBoxMode::Alignment::RIGHT;
 
-  const Font &font = *look.overlay_font;
+  const Font &font = *look.overlay.overlay_font;
   canvas.Select(font);
 
-  UPixelScalar padding = Layout::FastScale(4);
-  UPixelScalar height = font.GetHeight();
-  PixelScalar y = 0 + padding;
-  PixelScalar x = render_projection.GetScreenWidth() - padding;
+  unsigned padding = Layout::FastScale(4);
+  unsigned height = font.GetHeight();
+  PixelPoint p(render_projection.GetScreenSize().width - padding, padding);
 
   if (compass_visible)
     /* don't obscure the north arrow */
     /* TODO: obtain offset from CompassRenderer */
-    y += Layout::Scale(19) + Layout::FastScale(13);
+    p.y += Layout::Scale(19) + Layout::FastScale(13);
 
   if (terrain) {
-    short elevation = terrain->GetTerrainHeight(location);
-    if (!RasterBuffer::IsSpecial(elevation)) {
+    TerrainHeight elevation = terrain->GetTerrainHeight(location);
+    if (!elevation.IsSpecial()) {
       StaticString<64> elevation_long;
       elevation_long = _("Elevation: ");
-      elevation_long += FormatUserAltitude(fixed(elevation));
+      elevation_long += FormatUserAltitude(elevation.GetValue());
 
-      TextInBox(canvas, elevation_long, x, y, mode,
-                render_projection.GetScreenWidth(),
-                render_projection.GetScreenHeight());
+      TextInBox(canvas, elevation_long, p, mode,
+                render_projection.GetScreenSize());
 
-      y += height;
+      p.y += height;
     }
   }
 
@@ -131,11 +127,9 @@ GlueMapWindow::DrawPanInfo(Canvas &canvas) const
     if (newline != nullptr)
       *newline = _T('\0');
 
-    TextInBox(canvas, start, x, y, mode,
-              render_projection.GetScreenWidth(),
-              render_projection.GetScreenHeight());
+    TextInBox(canvas, start, p, mode, render_projection.GetScreenSize());
 
-    y += height;
+    p.y += height;
 
     if (newline == nullptr)
       break;
@@ -161,25 +155,25 @@ GlueMapWindow::DrawGPSStatus(Canvas &canvas, const PixelRect &rc,
     // early exit
     return;
 
-  PixelScalar x = rc.left + Layout::FastScale(2);
-  PixelScalar y = rc.bottom - Layout::FastScale(35);
-  icon->Draw(canvas, x, y);
+  PixelPoint p(rc.left + Layout::FastScale(2),
+               rc.bottom - Layout::FastScale(35));
+  icon->Draw(canvas, p);
 
-  x += icon->GetSize().cx + Layout::FastScale(4);
-  y = rc.bottom - Layout::FastScale(34);
+  p.x += icon->GetSize().width + Layout::FastScale(4);
+  p.y = rc.bottom - Layout::FastScale(34);
 
   TextInBoxMode mode;
   mode.shape = LabelShape::ROUNDED_BLACK;
 
-  const Font &font = *look.overlay_font;
+  const Font &font = *look.overlay.overlay_font;
   canvas.Select(font);
-  TextInBox(canvas, txt, x, y, mode, rc, nullptr);
+  TextInBox(canvas, txt, p, mode, rc, nullptr);
 }
 
 void
 GlueMapWindow::DrawFlightMode(Canvas &canvas, const PixelRect &rc) const
 {
-  PixelScalar offset = 0;
+  int offset = 0;
 
   // draw flight mode
   const MaskedIcon *bmp;
@@ -193,10 +187,11 @@ GlueMapWindow::DrawFlightMode(Canvas &canvas, const PixelRect &rc) const
   else
     bmp = &look.cruise_mode_icon;
 
-  offset += bmp->GetSize().cx + Layout::Scale(6);
+  offset += bmp->GetSize().width + Layout::Scale(6);
 
-  bmp->Draw(canvas, rc.right - offset,
-            rc.bottom - bmp->GetSize().cy - Layout::Scale(4));
+  bmp->Draw(canvas,
+            PixelPoint(rc.right - offset,
+                       rc.bottom - bmp->GetSize().height - Layout::Scale(4)));
 
   // draw flarm status
   if (!GetMapSettings().show_flarm_alarm_level)
@@ -221,10 +216,11 @@ GlueMapWindow::DrawFlightMode(Canvas &canvas, const PixelRect &rc) const
     break;
   };
 
-  offset += bmp->GetSize().cx + Layout::Scale(6);
+  offset += bmp->GetSize().width + Layout::Scale(6);
 
-  bmp->Draw(canvas, rc.right - offset,
-            rc.bottom - bmp->GetSize().cy - Layout::Scale(2));
+  bmp->Draw(canvas,
+            PixelPoint(rc.right - offset,
+                       rc.bottom - bmp->GetSize().height - Layout::Scale(2)));
 }
 
 void
@@ -244,8 +240,8 @@ GlueMapWindow::DrawFinalGlide(Canvas &canvas, const PixelRect &rc) const
     if (!task_stats.task_valid || !solution.IsOk() || !solution_mc0.IsDefined())
       return;
 
-    if (solution_mc0.SelectAltitudeDifference(glide_settings) < fixed(-1000) &&
-        solution.SelectAltitudeDifference(glide_settings) < fixed(-1000))
+    if (solution_mc0.SelectAltitudeDifference(glide_settings) < -1000 &&
+        solution.SelectAltitudeDifference(glide_settings) < -1000)
       return;
   }
 
@@ -265,45 +261,50 @@ GlueMapWindow::DrawVario(Canvas &canvas, const PixelRect &rc) const
                                 true); //NOTE: AVG enabled for now, make it configurable ;
 }
 
+void
+GlueMapWindow::SetBottomMargin(unsigned margin) noexcept
+{
+  if (margin == bottom_margin)
+    /* no change, don't redraw */
+    return;
+
+  bottom_margin = margin;
+  QuickRedraw();
+}
+
+void
+GlueMapWindow::SetBottomMarginFactor(unsigned margin_factor) noexcept
+{
+  if (margin_factor == 0) {
+    SetBottomMargin(0);
+    return;
+  }
+
+  PixelRect map_rect = GetClientRect();
+
+  if (map_rect.GetHeight() > map_rect.GetWidth()) {
+    SetBottomMargin(map_rect.bottom / margin_factor);
+  } else {
+    SetBottomMargin(0);
+  }
+}
+
 void
 GlueMapWindow::DrawMapScale(Canvas &canvas, const PixelRect &rc,
                             const MapWindowProjection &projection) const
 {
+
+  PixelRect scale_pos(rc.left, rc.top, rc.right, rc.bottom - bottom_margin);
+
+  RenderMapScale(canvas, projection, scale_pos, look.overlay);
+
   if (!projection.IsValid())
     return;
 
   StaticString<80> buffer;
 
-  fixed map_width = projection.GetScreenWidthMeters();
-
-  const Font &font = *look.overlay_font;
-  canvas.Select(font);
-  FormatUserMapScale(map_width, buffer.buffer(), true);
-  PixelSize text_size = canvas.CalcTextSize(buffer);
-
-  const PixelScalar text_padding_x = Layout::GetTextPadding();
-  const PixelScalar height = font.GetCapitalHeight()
-    + Layout::GetTextPadding();
-
-  PixelScalar x = 0;
-  look.map_scale_left_icon.Draw(canvas, 0, rc.bottom - height);
-
-  x += look.map_scale_left_icon.GetSize().cx;
-  canvas.DrawFilledRectangle(x, rc.bottom - height,
-                             x + 2 * text_padding_x + text_size.cx,
-                             rc.bottom, COLOR_WHITE);
-
-  canvas.SetBackgroundTransparent();
-  canvas.SetTextColor(COLOR_BLACK);
-  x += text_padding_x;
-  canvas.DrawText(x,
-                  rc.bottom - font.GetAscentHeight() - Layout::Scale(1),
-                  buffer);
-
-  x += text_padding_x + text_size.cx;
-  look.map_scale_right_icon.Draw(canvas, x, rc.bottom - height);
-
   buffer.clear();
+
   if (GetMapSettings().auto_zoom_enabled)
     buffer = _T("AUTO ");
 
@@ -334,21 +335,24 @@ GlueMapWindow::DrawMapScale(Canvas &canvas, const PixelRect &rc,
         _T("BALLAST %d LITERS "),
         (int)GetComputerSettings().polar.glide_polar_task.GetBallastLitres());
 
-  if (weather != nullptr && !weather->IsTerrain()) {
-    const RasterWeatherStore &ws = weather->GetStore();
-    const TCHAR *label = ws.GetItemInfo(weather->GetParameter()).label;
+  if (rasp_renderer != nullptr) {
+    const TCHAR *label = rasp_renderer->GetLabel();
     if (label != nullptr)
       buffer += gettext(label);
   }
 
   if (!buffer.empty()) {
-    int y = rc.bottom - height;
+
+    const Font &font = *look.overlay.overlay_font;
+    canvas.Select(font);
+    const int height = font.GetCapitalHeight()
+        + Layout::GetTextPadding();
 
     TextInBoxMode mode;
     mode.vertical_position = TextInBoxMode::VerticalPosition::ABOVE;
     mode.shape = LabelShape::OUTLINED;
 
-    TextInBox(canvas, buffer, 0, y, mode, rc, nullptr);
+    TextInBox(canvas, buffer, {0, scale_pos.bottom - height}, mode, rc, nullptr);
   }
 }
 
@@ -360,9 +364,8 @@ GlueMapWindow::DrawThermalEstimate(Canvas &canvas) const
     const MapWindowProjection &projection = render_projection;
     const ThermalLocatorInfo &thermal_locator = Calculated().thermal_locator;
     if (thermal_locator.estimate_valid) {
-      RasterPoint sc;
-      if (projection.GeoToScreenIfVisible(thermal_locator.estimate_location, sc)) {
-        look.thermal_source_icon.Draw(canvas, sc);
+      if (auto p = projection.GeoToScreenIfVisible(thermal_locator.estimate_location)) {
+        look.thermal_source_icon.Draw(canvas, *p);
       }
     }
   } else {
@@ -371,7 +374,7 @@ GlueMapWindow::DrawThermalEstimate(Canvas &canvas) const
 }
 
 void
-GlueMapWindow::RenderTrail(Canvas &canvas, const RasterPoint aircraft_pos)
+GlueMapWindow::RenderTrail(Canvas &canvas, const PixelPoint aircraft_pos)
 {
   unsigned min_time;
   switch(GetMapSettings().trail.length) {
@@ -394,7 +397,7 @@ GlueMapWindow::RenderTrail(Canvas &canvas, const RasterPoint aircraft_pos)
 }
 
 void
-GlueMapWindow::RenderTrackBearing(Canvas &canvas, const RasterPoint aircraft_pos)
+GlueMapWindow::RenderTrackBearing(Canvas &canvas, const PixelPoint aircraft_pos)
 {
   DrawTrackBearing(canvas, aircraft_pos, InCirclingMode());
 }
@@ -403,13 +406,13 @@ void
 GlueMapWindow::DrawThermalBand(Canvas &canvas, const PixelRect &rc) const
 {
   if (Calculated().task_stats.total.solution_remaining.IsOk() &&
-      Calculated().task_stats.total.solution_remaining.altitude_difference > fixed(50)
+      Calculated().task_stats.total.solution_remaining.altitude_difference > 50
       && GetDisplayMode() == DisplayMode::FINAL_GLIDE)
     return;
 
   PixelRect tb_rect;
   tb_rect.left = rc.left;
-  tb_rect.right = rc.left+Layout::Scale(20);
+  tb_rect.right = rc.left+Layout::Scale(25);
   tb_rect.top = Layout::Scale(2);
   tb_rect.bottom = (rc.bottom-rc.top)/5 - Layout::Scale(2);
 
@@ -440,10 +443,12 @@ GlueMapWindow::DrawStallRatio(Canvas &canvas, const PixelRect &rc) const
 {
   if (Basic().stall_ratio_available) {
     // JMW experimental, display stall sensor
-    fixed s = Clamp(Basic().stall_ratio, fixed(0), fixed(1));
-    PixelScalar m((rc.bottom - rc.top) * s * s);
+    auto s = Clamp(Basic().stall_ratio, 0., 1.);
+    int m = rc.GetHeight() * s * s;
+
+    const auto p = rc.GetBottomRight();
 
     canvas.SelectBlackPen();
-    canvas.DrawLine(rc.right - 1, rc.bottom - m, rc.right - 11, rc.bottom - m);
+    canvas.DrawLine(p.At(-1, -m), p.At(-11, -m));
   }
 }
