@@ -55,6 +55,12 @@ struct FlyingState
    */
   GeoPoint takeoff_location;
 
+  /**
+   * The altitude of the aircraft when it took off.  Only valid is
+   * takeoff_time is not negative.
+   */
+  double takeoff_altitude;
+
   /**
    * The time stamp when the aircraft released from towing.  This is
    * an estimate based on sink.  If the aircraft was never seen on
@@ -107,6 +113,14 @@ struct FlyingState
   /** Reset flying state as if never flown */
   void Reset();
 
+  /**
+   * Was a take-off recorded?  This validates the fields
+   * #takeoff_time, #takeoff_location and #takeoff_altitude.
+   */
+  bool HasTakenOff() const noexcept {
+    return takeoff_time >= 0;
+  }
+
   bool IsTowing() const {
     return flying && release_time < 0;
   }
