@@ -22,13 +22,13 @@ Copyright_License {
 */
 
 #include "NMEA/Info.hpp"
-#include "system/Clock.hpp"
 #include "Atmosphere/AirDensity.hpp"
+#include "time/Cast.hxx"
 
 void
 NMEAInfo::UpdateClock()
 {
-  clock = MonotonicClockFloat();
+  clock = ToFloatSeconds(std::chrono::steady_clock::now().time_since_epoch());
 }
 
 BrokenDateTime
@@ -41,7 +41,7 @@ NMEAInfo::GetDateTimeAt(double other_time) const
     return BrokenDateTime(BrokenDate::Invalid(),
                           BrokenTime::FromSecondOfDayChecked(int(other_time)));
 
-  return date_time_utc + int(other_time - time);
+  return date_time_utc + std::chrono::duration_cast<std::chrono::system_clock::duration>(FloatDuration{other_time - time});
 }
 
 void
@@ -93,8 +93,8 @@ NMEAInfo::ProvideTrueAirspeed(double tas)
 {
   auto any_altitude = GetAnyAltitude();
 
-  if (any_altitude.first)
-    ProvideTrueAirspeedWithAltitude(tas, any_altitude.second);
+  if (any_altitude)
+    ProvideTrueAirspeedWithAltitude(tas, *any_altitude);
   else
     /* no altitude; dirty fallback */
     ProvideBothAirspeeds(tas, tas);
@@ -105,8 +105,8 @@ NMEAInfo::ProvideIndicatedAirspeed(double ias)
 {
   auto any_altitude = GetAnyAltitude();
 
-  if (any_altitude.first)
-    ProvideIndicatedAirspeedWithAltitude(ias, any_altitude.second);
+  if (*any_altitude)
+    ProvideIndicatedAirspeedWithAltitude(ias, *any_altitude);
   else
     /* no altitude; dirty fallback */
     ProvideBothAirspeeds(ias, ias);
