@@ -24,7 +24,7 @@ Copyright_License {
 #ifndef XCSOAR_VALIDITY_HPP
 #define XCSOAR_VALIDITY_HPP
 
-#include "util/Compiler.h"
+#include "time/FloatDuration.hxx"
 
 #include <chrono>
 #include <type_traits>
@@ -41,7 +41,6 @@ class Validity {
   static constexpr int BITS = 6;
 
   using Duration = std::chrono::duration<uint32_t, std::ratio<1, 1 << BITS>>;
-  using FloatDuration = std::chrono::duration<double>;
 
   Duration last;
 
@@ -83,6 +82,15 @@ public:
     last = Import(now);
   }
 
+  /**
+   * Cast this object to an integer.  This integer should only be used
+   * for equality comparisons with other integers from this method,
+   * e.g. to check if a redraw is needed.
+   */
+  auto ToInteger() const noexcept {
+    return last.count();
+  }
+
   /**
    * Checks if the time stamp has expired, and calls clear() if so.
    *
