@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,14 +25,15 @@ Copyright_License {
 #define XCSOAR_TRACKING_SKYLINES_TRAFFIC_HPP
 
 #include "Geo/GeoPoint.hpp"
-#include "Thread/Mutex.hpp"
-#include "Util/tstring.hpp"
-#include "Compiler.h"
+#include "thread/Mutex.hxx"
+#include "util/tstring.hpp"
+#include "util/Compiler.h"
 
 #include <map>
 #include <list>
+#include <chrono>
 
-#include <stdint.h>
+#include <cstdint>
 
 namespace SkyLinesTracking {
 
@@ -74,6 +75,19 @@ struct Data {
     :time_of_day_ms(_time), a(_a), b(_b) {}
   };
 
+  struct Thermal {
+    std::chrono::steady_clock::time_point received_time;
+
+    AGeoPoint bottom_location, top_location;
+
+    double lift;
+
+    Thermal() = default;
+    Thermal(const AGeoPoint &_bottom, const AGeoPoint &_top, double _lift)
+      :received_time(std::chrono::steady_clock::now()),
+       bottom_location(_bottom), top_location(_top), lift(_lift) {}
+  };
+
   mutable Mutex mutex;
 
   std::map<uint32_t, Traffic> traffic;
@@ -86,6 +100,8 @@ struct Data {
 
   std::list<Wave> waves;
 
+  std::list<Thermal> thermals;
+
   gcc_pure
   bool IsUserKnown(uint32_t id) const {
     return user_names.find(id) != user_names.end();
