@@ -20,10 +20,11 @@ Copyright_License {
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 }
 */
+
 #ifndef LOCALTIME_H
 #define LOCALTIME_H
 
-#include "util/Compiler.h"
+#include <chrono>
 
 class RoughTimeDelta;
 
@@ -32,7 +33,11 @@ class RoughTimeDelta;
  * @param d UTC time in seconds
  * @param utc_offset Offset in second
  */
-gcc_pure
+[[gnu::pure]]
+std::chrono::seconds
+TimeLocal(std::chrono::seconds d, RoughTimeDelta utc_offset) noexcept;
+
+[[gnu::pure]]
 unsigned
 TimeLocal(int d, RoughTimeDelta utc_offset);
 
